__version__ = "0.0.1"

# Headers
# values
CONTENT_TYPE = "application/json"
APPLICATION = "developer_api"

# keys
ACCESS_TOKEN_HEADER = "X-PW-AccessToken"
APPLICATION_HEADER = "X-PW-Application"
EMAIL_HEADER = "X-PW-UserEmail"


BASE_URL = "https://api.prosperworks.com/developer_api/v1/"
